@@ -345,7 +345,7 @@ class MainWindow(hildon.StackableWindow):
         show_information(self, _('Updated "%s"') % show.name)
 
     def _update_show_art(self, series_manager, show):
-        self.shows_view.update(show)
+        self.shows_view.update_art(show)
 
     def _about_menu_clicked_cb(self, menu):
         about_dialog = AboutDialog(self)
@@ -444,10 +444,10 @@ class ShowsSelectView(gtk.TreeView):
         self.tree_filter = self.tree_model.filter_new()
         self.set_model(self.tree_filter)
         self.append_column(column)
-        self.sort()
 
     def set_shows(self, shows):
         self.tree_model.add_shows(shows)
+        gobject.idle_add(self.sort)
 
     def get_show_from_path(self, path):
         return self.get_model()[path][ShowListStore.SHOW_COLUMN]
@@ -467,6 +467,10 @@ class ShowsSelectView(gtk.TreeView):
             self.tree_model.update(show)
             self.sort()
 
+    def update_art(self, show = None):
+        if self.tree_model:
+            self.tree_model.update_pixmaps(show)
+
     def sort(self):
         shows_sort_order = Settings().getConf(Settings.SHOWS_SORT)
         if shows_sort_order == Settings.RECENT_EPISODE:
@@ -485,19 +489,22 @@ class ShowListStore(gtk.ListStore):
     def __init__(self):
         super(ShowListStore, self).__init__(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT, str, gobject.TYPE_PYOBJECT)
         self.cached_pixbufs = {}
+        self.downloading_pixbuf = get_downloading_pixbuf()
         self.set_sort_func(self.NEXT_EPISODE_COLUMN, self._sort_func)
 
     def add_shows(self, shows):
         self.clear()
         for show in shows:
-            row = {self.IMAGE_COLUMN: None,
-                   self.INFO_COLUMN: saxutils.escape(show.name),
+            escaped_name = saxutils.escape(show.name)
+            row = {self.IMAGE_COLUMN: self.downloading_pixbuf,
+                   self.INFO_COLUMN: escaped_name,
                    self.SHOW_COLUMN: show,
                    self.SEARCH_COLUMN: saxutils.escape(show.name).lower(),
                    self.NEXT_EPISODE_COLUMN: None
                   }
             self.append(row.values())
         self.update(None)
+        self.update_pixmaps()
 
     def update(self, show):
         iter = self.get_iter_first()
@@ -509,13 +516,14 @@ class ShowListStore(gtk.ListStore):
 
     def _update_iter(self, iter):
         show = self.get_value(iter, self.SHOW_COLUMN)
-        pixbuf = self.get_value(iter, self.IMAGE_COLUMN)
         info = show.get_episodes_info()
         info_markup = show.get_info_markup(info)
         self.set_value(iter, self.INFO_COLUMN, info_markup)
         self.set_value(iter, self.NEXT_EPISODE_COLUMN,
                        info['next_episode'])
         self.set_value(iter, self.SEARCH_COLUMN, show.name.lower())
+
+    def _load_pixmap_async(self, show, pixbuf):
         if pixbuf_is_cover(pixbuf):
             return
         if show.image and os.path.isfile(show.image):
@@ -528,14 +536,37 @@ class ShowListStore(gtk.ListStore):
                 except:
                     pixbuf = get_placeholder_pixbuf()
             self.cached_pixbufs[show.image] = pixbuf
-            self.set_value(iter, self.IMAGE_COLUMN, pixbuf)
         elif show.downloading_show_image:
-            pixbuf = get_downloading_pixbuf()
-            self.set_value(iter, self.IMAGE_COLUMN, pixbuf)
+            pixbuf = self.downloading_pixbuf
         else:
             pixbuf = get_placeholder_pixbuf()
+        return pixbuf
+
+    def _load_pixmap_async_finished(self, show, pixbuf, error):
+        if error or not pixbuf:
+            return
+        iter = self._get_iter_for_show(show)
+        if iter:
             self.set_value(iter, self.IMAGE_COLUMN, pixbuf)
 
+    def update_pixmaps(self, show = None):
+        iter = self.get_iter_first()
+        async_worker = AsyncWorker()
+        while iter:
+            current_show = self.get_value(iter, self.SHOW_COLUMN)
+            same_show = show == current_show
+            if show is None or same_show:
+                pixbuf = self.get_value(iter, self.IMAGE_COLUMN)
+                async_item = AsyncItem(self._load_pixmap_async,
+                                       (current_show, pixbuf),
+                                       self._load_pixmap_async_finished,
+                                       (current_show,))
+                async_worker.queue.put((0, async_item))
+                if same_show:
+                    break
+            iter = self.iter_next(iter)
+        async_worker.start()
+
     def _sort_func(self, model, iter1, iter2):
         episode1 = model.get_value(iter1, self.NEXT_EPISODE_COLUMN)
         episode2 = model.get_value(iter2, self.NEXT_EPISODE_COLUMN)
@@ -553,6 +584,17 @@ class ShowListStore(gtk.ListStore):
             return -1
         return 1
 
+    def _get_iter_for_show(self, show):
+        if not show:
+            return None
+        iter = self.get_iter_first()
+        while iter:
+            current_show = self.get_value(iter, self.SHOW_COLUMN)
+            if show == current_show:
+                break
+            iter = self.iter_next(iter)
+        return iter
+
 class SeasonsView(hildon.StackableWindow):
 
     def __init__(self, settings, series_manager, connection_manager, show):
@@ -584,6 +626,12 @@ class SeasonsView(hildon.StackableWindow):
         seasons_area.add(self.seasons_select_view)
         self.add(seasons_area)
 
+        if self.settings.getConf(self.settings.SEASONS_ORDER_CONF_NAME) == \
+           self.settings.ASCENDING_ORDER:
+            self._sort_ascending_cb(None)
+        else:
+            self._sort_descending_cb(None)
+
         self.request = None
         self._update_menu_visibility()
 
@@ -608,6 +656,23 @@ class SeasonsView(hildon.StackableWindow):
     def _create_menu(self):
         menu = hildon.AppMenu()
 
+        button_asc = hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        button_asc.set_mode(False)
+        button_asc.set_label(_('A-Z'))
+        menu.add_filter(button_asc)
+        button_desc = hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                            group = button_asc)
+        button_desc.set_mode(False)
+        button_desc.set_label(_('Z-A'))
+        menu.add_filter(button_desc)
+        if self.settings.getConf(Settings.SEASONS_ORDER_CONF_NAME) == \
+                Settings.ASCENDING_ORDER:
+            button_asc.set_active(True)
+        else:
+            button_desc.set_active(True)
+        button_asc.connect('clicked', self._sort_ascending_cb)
+        button_desc.connect('clicked', self._sort_descending_cb)
+
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         button.set_label(_('Info'))
         button.connect('clicked', self._show_info_cb)
@@ -744,6 +809,16 @@ class SeasonsView(hildon.StackableWindow):
         else:
             self.update_menu.hide()
 
+    def _sort_ascending_cb(self, button):
+        self.seasons_select_view.sort_ascending()
+        self.settings.setConf(self.settings.SEASONS_ORDER_CONF_NAME,
+                              self.settings.ASCENDING_ORDER)
+
+    def _sort_descending_cb(self, button):
+        self.seasons_select_view.sort_descending()
+        self.settings.setConf(self.settings.SEASONS_ORDER_CONF_NAME,
+                              self.settings.DESCENDING_ORDER)
+
 class SeasonsDeleteView(DeleteView):
 
     def __init__(self, series_manager, seasons_select_view):
@@ -763,10 +838,10 @@ class SeasonsDeleteView(DeleteView):
         if not paths:
             show_information(self, _('Please select one or more seasons'))
             return
-        for path in paths:
-            self.seasons_select_view.show.delete_season(
-                 model[path][SeasonListStore.SEASON_COLUMN])
-            model.remove(model.get_iter(path))
+        seasons = [model[path][SeasonListStore.SEASON_COLUMN] for path in paths]
+        for season in seasons:
+            self.seasons_select_view.show.delete_season(season)
+            model.delete_season(season)
         self.destroy()
 
 class SeasonListStore(gtk.ListStore):
@@ -778,7 +853,7 @@ class SeasonListStore(gtk.ListStore):
     def __init__(self, show):
         super(SeasonListStore, self).__init__(gtk.gdk.Pixbuf,
                                               str,
-                                              gobject.TYPE_PYOBJECT)
+                                              str)
         self.show = show
 
     def add(self, season_list):
@@ -801,6 +876,14 @@ class SeasonListStore(gtk.ListStore):
             self._update_iter(iter)
             iter = self.iter_next(iter)
 
+    def delete_season(self, season):
+        iter = self.get_iter_first()
+        while iter:
+            if self.get_value(iter, self.SEASON_COLUMN) == season:
+                self.remove(iter)
+                break
+            iter = self.iter_next(iter)
+
     def _update_iter(self, iter):
         season = self.get_value(iter, self.SEASON_COLUMN)
         info = self.show.get_season_info_markup(season)
@@ -854,6 +937,25 @@ class SeasonSelectView(gtk.TreeView):
         if model:
             model.update()
 
+    def _sort_func(self, model, iter1, iter2):
+        season1 = model.get_value(iter1, SeasonListStore.SEASON_COLUMN)
+        season2 = model.get_value(iter2, SeasonListStore.SEASON_COLUMN)
+        if season1 == None or season2 == None:
+            return 0
+        if season1 < season2:
+            return -1
+        return 1
+
+    def sort_descending(self):
+        model = self.get_model()
+        model.set_sort_column_id(SeasonListStore.SEASON_COLUMN,
+                                 gtk.SORT_DESCENDING)
+
+    def sort_ascending(self):
+        model = self.get_model()
+        model.set_sort_column_id(SeasonListStore.SEASON_COLUMN,
+                                 gtk.SORT_ASCENDING)
+
 class NewShowDialog(gtk.Dialog):
 
     def __init__(self, parent):
@@ -1177,10 +1279,11 @@ class EpisodeListStore(gtk.ListStore):
 
 
     def update(self):
-        iter = self.get_iter_first()
+        iter = self.get_iter_root()
         while iter:
+            next_iter = self.iter_next(iter)
             self.update_iter(iter)
-            iter = self.iter_next(iter)
+            iter = next_iter
 
     def update_iter(self, iter):
         episode = self.get_value(iter, self.EPISODE_COLUMN)
