@@ -24,7 +24,7 @@ import gtk
 
 SF_NAME = 'SeriesFinale'
 SF_COMPACT_NAME = 'seriesfinale'
-SF_VERSION = '0.6.7'
+SF_VERSION = '0.6.8'
 SF_DESCRIPTION = 'SeriesFinale is a TV series browser and tracker application'
 SF_URL = 'http://www.igalia.com'
 SF_COPYRIGHT = 'Copyright © 2010-2011 Igalia S. L.'
@@ -52,8 +52,8 @@ SF_DB_FILE = SF_CONF_FOLDER + '/%s' % 'series.db'
 SF_PID_FILE = '/tmp/seriesfinale.pid'
 _XDG_DATA_HOME = os.getenv('XDG_DATA_HOME') or ''
 _XDG_DATA_HOME = _XDG_DATA_HOME.split(':')[0]
-_DATA_DIR_PREFIX = _XDG_DATA_HOME or HOME_PATH
-DATA_DIR = os.path.join(_DATA_DIR_PREFIX, '.local', 'share', SF_COMPACT_NAME)
+_DATA_DIR_PREFIX = _XDG_DATA_HOME or os.path.join(HOME_PATH, '.local', 'share')
+DATA_DIR = os.path.join(_DATA_DIR_PREFIX, SF_COMPACT_NAME)
 if not os.path.exists(DATA_DIR):
     try:
         os.makedirs(DATA_DIR)
